#include "alternate-gpio-init.hpp"
#include "gpio.hpp"

#ifndef ALTERNATE_GPIO_H
#define ALTERNATE_GPIO_H

namespace stm32f072xb {

template <Port T>
class AlternateGpio : public Gpio<T> {
   public:
	AlternateGpio(const AlternateGpioInit& gpioInit) : Gpio<T>(gpioInit) {
		CLEAR_BIT(this->getGpio()->OTYPER, 1 << (this->getPin()));
		CLEAR_BIT(this->getGpio()->MODER, 0x3UL << (this->getPin() * 2));
		SET_BIT(this->getGpio()->MODER, 0x2UL << (this->getPin() * 2));

		uint32_t position = 0;
		uint8_t level = 0;

		if (int(gpioInit.pin) < (int)GpioPin::P8) {
			position = 4 * (int)gpioInit.pin;
			level = 0;
		}

		else {
			position = 4 * (((int)gpioInit.pin) % 8);
			level = 1;
		}

		SET_BIT(this->getGpio()->AFR[level], (int)gpioInit.alternateFunction << position);
	}
};

}  // namespace stm32f072xb

#endif
